import sys
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

if not(len(sys.argv)==3) :
        print("expected input : gff files of the two transfer programms : Liftoff and GrAnnoT")
        sys.exit(1)
elif (sys.argv[1]=="-h") :
    print("expected input : gff files of the two transfer programms : Liftoff and GrAnnoT")
    sys.exit(1)

liftoff_file=sys.argv[1]
grannot_file=sys.argv[2]


# build a dictionnary with the GrAnnoT coordinates conversions

GrAnnoT = open(grannot_file,'r')
lines=GrAnnoT.readlines()
GrAnnoT.close()

grannot_gff={}
transpo=0
non_transpo=0
for line in lines:
    #if line.split()[2]=="gene":
    line=line.split()
    feature=line[-1].split(';')[0].split('=')[1]
    chr=line[0]#.split("_")[0]
    start=int(line[3])
    stop=int(line[4])
    cov=float(line[-1].split(';')[-2].split('=')[1])
    id=float(line[-1].split(';')[-1].split('=')[1])
    pos=list([chr,start,stop,cov,id])
    if feature not in grannot_gff:
        grannot_gff[feature]=[]
    grannot_gff[feature].append(pos) # feat_id -> [chr, start, stop, cov, seq_id]

        # if 'ranspo' in line[8]:
        #     transpo+=1
        # else:
        #     non_transpo+=1

print(len(grannot_gff),"features transfered with GrAnnoT\n")


def search_cov_id(annot):
    annot=annot.split(";")
    cov=''
    id=''
    for elem in annot:
        elem=elem.split("=")
        if elem[0]=="coverage":
            cov=float(elem[1])
        if elem[0]=="sequence_ID":
            id=float(elem[1])
    return (cov,id)


# build a dictionnary with the Liftoff alignements

liftoff = open(liftoff_file,'r')
lines=liftoff.readlines()
liftoff.close()

liftoff_gff={}
for line in lines:
    if (line[0]!="#"):# and (line.split()[2]=="gene"):
        line=line.split()
        feature=line[8].split(';')[0].split('=')[1]
        chr=line[0]#.split("#")[2]
        start=int(line[3])
        stop=int(line[4])
        (cov,seq_id)=search_cov_id(line[8])
        pos=list([chr,start,stop,cov,seq_id])
        if feature not in liftoff_gff:
            liftoff_gff[feature]=[]
        liftoff_gff[feature].append(pos) # feat_id -> [chr, start, stop, cov, seq_id]

        #if feature not in grannot_gff:
        #    print("\t".join(line))

print(len(liftoff_gff), "features transfered with Liftoff")


from tqdm import tqdm

# intersection between the two transfers :
common=set();only_liftoff=set();only_grannot=set();too_much_occ=set()
for feat_liftoff in liftoff_gff:
    if len(liftoff_gff[feat_liftoff])==1:
        if feat_liftoff in grannot_gff:
            if len(grannot_gff[feat_liftoff])==1:
                common.add(feat_liftoff)
            else:
                too_much_occ.add(feat_liftoff)
        else:
            only_liftoff.add(feat_liftoff)
    else:
        too_much_occ.add(feat_liftoff)

for feat_grannot in grannot_gff:
    if len(grannot_gff[feat_grannot])==1:
        if feat_grannot in liftoff_gff:
            if len(liftoff_gff[feat_grannot])==1:
                common.add(feat_grannot)
            else:
                too_much_occ.add(feat_grannot)
        else:
            only_grannot.add(feat_grannot)
    else:
        too_much_occ.add(feat_grannot)

print(len(common),"features were transfered by both programs")
print(len(too_much_occ),"features with more than one occ in one of the programs")
print(len(only_liftoff),"features were transfered only by Liftoff")
print(len(only_grannot),"features were transfered only by GrAnnoT\n")

def close(posa,posb,dista,distb):
    if (posa[0]==posb[0]):# same chr
        if ((abs(posa[1]-posb[1])+abs(posa[2]-posb[2]))>=dista) and ((abs(posa[1]-posb[1])+abs(posa[2]-posb[2]))<=distb):
            return True
        else:
            return False
    else:
        return "chr"


def compare_feat(feat_l,feat_g,dista,distb): # [chr, start, stop, cov, seq_id]
    close_elem=0
    diff_chr=0
    for elem_l in feat_l:
        if close(feat_g[0],elem_l,dista,distb)=='chr':
            diff_chr+=1
        for elem_g in feat_g:
            if close(elem_g,elem_l,dista,distb)==True:
                close_elem+=1
    return [close_elem,diff_chr]
                
import math
# common transfers
[diff_chr,same_pos,diff_pos,close_0,close_ish,close_ishish,not_close,dist]=[0,0,0,0,0,0,0,[]]
list_diff_pos=list()
for feat in tqdm(common):
    same_pos+=compare_feat(grannot_gff[feat],liftoff_gff[feat],0,0)[0]
    close_0+=compare_feat(grannot_gff[feat],liftoff_gff[feat],1,9)[0]
    close_ish+=compare_feat(grannot_gff[feat],liftoff_gff[feat],10,49)[0]
    close_ishish+=compare_feat(grannot_gff[feat],liftoff_gff[feat],50,99)[0]
    not_close+=compare_feat(grannot_gff[feat],liftoff_gff[feat],100,math.inf)[0]
    diff_chr+=compare_feat(grannot_gff[feat],liftoff_gff[feat],0,0)[1]

    # if liftoff_gff[feat][0]==grannot_gff[feat][0]: # check if same chromosome 
    #     #dist.append(abs(liftoff_gff[feat][1]-grannot_gff[feat][1])+abs(liftoff_gff[feat][2]-grannot_gff[feat][2]))
    #     if (liftoff_gff[feat][1]==grannot_gff[feat][1]) & (liftoff_gff[feat][2]==grannot_gff[feat][2]): # exact same position
    #         same_pos+=1
    #     else: 
    #         diff_pos+=1
    #         if (abs(liftoff_gff[feat][1]-grannot_gff[feat][1])<10) & (abs(liftoff_gff[feat][2]-grannot_gff[feat][2])<10): # start and stop close
    #             close_0+=1
    #         elif (abs(liftoff_gff[feat][1]-grannot_gff[feat][1])<50) & (abs(liftoff_gff[feat][2]-grannot_gff[feat][2])<50): # start and stop not to far
    #             close_ish+=1
    #         elif (abs(liftoff_gff[feat][1]-grannot_gff[feat][1])<100) & (abs(liftoff_gff[feat][2]-grannot_gff[feat][2])<100): # start and stop quite far
    #             close_ishish+=1
    #         else:
    #             not_close+=1

    # else:
    #     diff_chr+=1


# sns.set_theme(style="darkgrid")
# sns
# dist_filter=[i for i in dist if ((i > 0) and (i<1001))]
# data_dist={'distance':dist_filter}

# dfff = pd.DataFrame(data_dist)
# #plt.xlim(0,1000)
# sns.histplot(data=dfff,x='distance',color="skyblue",label="distance",bins=1000)
# plt.savefig('hist_dist_1-1000.png')
# plt.show()

diff_pos=close_0+close_ish+close_ishish+not_close+diff_chr

print(same_pos,"features are at the exact same position between the two transfers")
print(diff_pos,"features are not in the exact same position between the two transfers. Among those :")
print(close_0,"have their start and stop positions less than 10 bases away")
print(close_ish,"have their start and stop positions between 11 and 50 bases away")
print(close_ishish,"have their start and stop positions between 51 and 100 bases away")
print(not_close,"have their start and stop positions more that 100 bases away")
print(diff_chr,'features are not on the same chromosome between the two transfers\n\n')

exit()


# average sequence_identity and coverage for liftoff's transfers

coverage=list();sequence_id=list()
for feat in common:
    coverage.append(liftoff_gff[feat][3])
    sequence_id.append(liftoff_gff[feat][4])



# sns.set_theme(style="darkgrid")
# sns
# fig, axs = plt.subplots(2, 2, figsize=(7, 7))
# axs[0,0].set_xlim(0.8,1)
# axs[0,1].set_xlim(0.8,1)
# axs[1,0].set_xlim(0.8,1)
# axs[1,1].set_xlim(0.8,1)
# dataa={'coverage liftoff':coverage,'seq_id liftoff':sequence_id}
# df = pd.DataFrame(dataa)
# sns.histplot(data=df,x='coverage liftoff',color="skyblue",label="coverage liftoff",ax=axs[0, 0],bins=100)
# sns.histplot(data=df,x='seq_id liftoff',color="red",label="sequence_id liftoff",ax=axs[0, 1],bins=100)
# #plt.show()


print("For the transfers made with Liftoff common with GrAnnoT :")
print("the average coverage of the features is",sum(coverage)/len(coverage))
print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))

coverage=list();sequence_id=list()
for feat in only_liftoff:
    coverage.append(liftoff_gff[feat][3])
    sequence_id.append(liftoff_gff[feat][4])
print("\nFor the transfers made with Liftoff not found with GrAnnoT :")
print("the average coverage of the features is",sum(coverage)/len(coverage))
print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))


# average size difference and variation number for GrAnnoT's transfers
coverage=list();sequence_id=list()
for feat in common:
    coverage.append(grannot_gff[feat][3])
    sequence_id.append(grannot_gff[feat][4])

# datab={'coverage grannot':coverage,'seq_id grannot':sequence_id}
# dff = pd.DataFrame(datab)
# sns.histplot(data=dff,x='coverage grannot',color="gold",label="coverage grannot",ax=axs[1, 0],bins=100)
# sns.histplot(data=dff,x='seq_id grannot',color="teal",label="sequence_id grannot",ax=axs[1, 1],bins=100)
# plt.savefig('compare_common.png')


print("\n\nFor the transfers made with GrAnnoT common with Liftoff :")
print("the average coverage of the features is",sum(coverage)/len(coverage))
print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))

coverage=list();sequence_id=list()
for feat in only_grannot:
    coverage.append(grannot_gff[feat][3])
    sequence_id.append(grannot_gff[feat][4])
print("\nFor the transfers made with GrAnnoT not found with Liftoff :")
print("the average coverage of the features is",sum(coverage)/len(coverage))
print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))
