import sys
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

if not(len(sys.argv)==3) :
        print("expected input : gff files of the two transfer programms : Liftoff and GrAnnoT")
        sys.exit(1)
elif (sys.argv[1]=="-h") :
    print("expected input : gff files of the two transfer programms : Liftoff and GrAnnoT")
    sys.exit(1)

liftoff_file=sys.argv[1]
grannot_file=sys.argv[2]


# build a dictionnary with the GrAnnoT coordinates conversions

GrAnnoT = open(grannot_file,'r')
lines=GrAnnoT.readlines()
GrAnnoT.close()

grannot_gff={}
transpo=0
non_transpo=0
for line in lines:
    if line.split()[2]=="gene":
        line=line.split()
        feature=line[8].split(';')[0].split('=')[1]
        #chr=line[0]
        #chr='Chr'+line[0][line[0].find("chromosome")+10:]
        chr=line[0].split("_")[0]
        start=int(line[3])
        stop=int(line[4])
        cov=float(line[8].split(';')[-2].split('=')[1])
        id=float(line[8].split(';')[-1].split('=')[1])
        pos=list([chr,start,stop,cov,id])
        if feature in grannot_gff:
            if cov+id>grannot_gff[feature][3]+grannot_gff[feature][4]:
                grannot_gff[feature]=pos
        else:
            grannot_gff[feature]=pos # feat_id -> [chr, start, stop, cov, seq_id]

            if 'ranspo' in line[8]:
                transpo+=1
            else:
                non_transpo+=1

print(len(grannot_gff),"features transfered with GrAnnoT")



# build a dictionnary with the Liftoff alignements

liftoff = open(liftoff_file,'r')
lines=liftoff.readlines()
liftoff.close()

liftoff_gff={}
for line in lines:
    if (line[0]!="#") and (line.split()[2]=="gene"):
        line=line.split()
        feature=line[8].split(';')[0].split('=')[1]
        chr=line[0]
        start=int(line[3])
        stop=int(line[4])
        cov=float(line[8].split(';')[3].split('=')[1])
        seq_id=float(line[8].split(';')[4].split('=')[1])
        pos=list([chr,start,stop,cov,seq_id])
        liftoff_gff[feature]=pos # feat_id -> [chr, start, stop, cov, seq_id]

        #if feature not in grannot_gff:
        #    print("\t".join(line))

print(len(liftoff_gff), "features transfered with Liftoff")




# intersection between the two transfers :
common=list();only_liftoff=list();only_grannot=list()
for feat_liftoff in liftoff_gff:
    if feat_liftoff in grannot_gff:
        if feat_liftoff not in common:
            common.append(feat_liftoff)
    else:
        only_liftoff.append(feat_liftoff)

for feat_grannot in grannot_gff:
    if feat_grannot in liftoff_gff:
        if feat_grannot not in common:
            common.append(feat_grannot)
    else:
        only_grannot.append(feat_grannot)

print(len(common),"features were transfered by both programs")
print(len(only_liftoff),"features were transfered only by Liftoff")
print(len(only_grannot),"features were transfered only by GrAnnoT\n")


# common transfers
[diff_chr,same_pos,diff_pos,close,close_ish,close_ishish,not_close,dist]=[0,0,0,0,0,0,0,[]]
list_diff_pos=list()
for feat in common:
    if liftoff_gff[feat][0]==grannot_gff[feat][0]: # check if same chromosome 
        dist.append(abs(liftoff_gff[feat][1]-grannot_gff[feat][1])+abs(liftoff_gff[feat][2]-grannot_gff[feat][2]))
        if (liftoff_gff[feat][1]==grannot_gff[feat][1]) & (liftoff_gff[feat][2]==grannot_gff[feat][2]): # exact same position
            same_pos+=1
        else: 
            diff_pos+=1
            if (abs(liftoff_gff[feat][1]-grannot_gff[feat][1])<10) & (abs(liftoff_gff[feat][2]-grannot_gff[feat][2])<10): # start and stop close
                close+=1
            elif (abs(liftoff_gff[feat][1]-grannot_gff[feat][1])<50) & (abs(liftoff_gff[feat][2]-grannot_gff[feat][2])<100): # start and stop not to far
                close_ish+=1
            elif (abs(liftoff_gff[feat][1]-grannot_gff[feat][1])<100) & (abs(liftoff_gff[feat][2]-grannot_gff[feat][2])<1000): # start and stop quite far
                close_ishish+=1
            else:
                not_close+=1

    else:
        diff_chr+=1
diff_pos+=diff_chr

plot=False
if plot:
    sns.set_theme(style="darkgrid")
    sns
    dist_filter=[i for i in dist if ((i > 0) and (i<1001))]
    data_dist={'distance':dist_filter}

    dfff = pd.DataFrame(data_dist)
    #plt.xlim(0,1000)
    sns.histplot(data=dfff,x='distance',color="skyblue",label="distance",bins=1000)
    plt.savefig('hist_dist_1-1000.png')
    plt.show()


print(same_pos,"features are at the exact same position between the two transfers")
print(diff_pos,"features are not in the exact same position between the two transfers. Among those :")
print(close,"have their start and stop positions less than 10 bases away")
print(close_ish,"have their start and stop positions between 11 and 100 bases away")
print(close_ishish,"have their start and stop positions between 101 and 1000 bases away")
print(not_close,"have their start and stop positions more that 1000 bases away")
print(diff_chr,"are not on the same chromosome\n\n")


# average sequence_identity and coverage for liftoff's transfers

coverage=list();sequence_id=list()
for feat in common:
    coverage.append(liftoff_gff[feat][3])
    sequence_id.append(liftoff_gff[feat][4])



# sns.set_theme(style="darkgrid")
# sns
# fig, axs = plt.subplots(2, 2, figsize=(7, 7))
# axs[0,0].set_xlim(0.8,1)
# axs[0,1].set_xlim(0.8,1)
# axs[1,0].set_xlim(0.8,1)
# axs[1,1].set_xlim(0.8,1)
# dataa={'coverage liftoff':coverage,'seq_id liftoff':sequence_id}
# df = pd.DataFrame(dataa)
# sns.histplot(data=df,x='coverage liftoff',color="skyblue",label="coverage liftoff",ax=axs[0, 0],bins=100)
# sns.histplot(data=df,x='seq_id liftoff',color="red",label="sequence_id liftoff",ax=axs[0, 1],bins=100)
# #plt.show()

stat_cov_id=False
if stat_cov_id:
    print("For the transfers made with Liftoff common with GrAnnoT :")
    print("the average coverage of the features is",sum(coverage)/len(coverage))
    print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))

    coverage=list();sequence_id=list()
    for feat in only_liftoff:
        coverage.append(liftoff_gff[feat][3])
        sequence_id.append(liftoff_gff[feat][4])
    print("\nFor the transfers made with Liftoff not found with GrAnnoT :")
    print("the average coverage of the features is",sum(coverage)/len(coverage))
    print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))


    # average size difference and variation number for GrAnnoT's transfers
    coverage=list();sequence_id=list()
    for feat in common:
        coverage.append(grannot_gff[feat][3])
        sequence_id.append(grannot_gff[feat][4])

    # datab={'coverage grannot':coverage,'seq_id grannot':sequence_id}
    # dff = pd.DataFrame(datab)
    # sns.histplot(data=dff,x='coverage grannot',color="gold",label="coverage grannot",ax=axs[1, 0],bins=100)
    # sns.histplot(data=dff,x='seq_id grannot',color="teal",label="sequence_id grannot",ax=axs[1, 1],bins=100)
    # plt.savefig('compare_common.png')


    print("\n\nFor the transfers made with GrAnnoT common with Liftoff :")
    print("the average coverage of the features is",sum(coverage)/len(coverage))
    print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))

    coverage=list();sequence_id=list()
    for feat in only_grannot:
        coverage.append(grannot_gff[feat][3])
        sequence_id.append(grannot_gff[feat][4])
    print("\nFor the transfers made with GrAnnoT not found with Liftoff :")
    print("the average coverage of the features is",sum(coverage)/len(coverage))
    print("the average sequence identity of the features is",sum(sequence_id)/len(sequence_id))
